#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# unlAnt: ASD of UNL Graph
#------------------------------------------------------------------------------
# --
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

import pydot as dot

import asd.urdf as urdf


#==============================================================================
# Parameters
#==============================================================================

UNL_RDF_SCHEMA = "https://unl.tetras-libre.fr/rdf/schema/"
UNL_RDF_BASE_URI = "https://unl.tetras-libre.fr/rdf/dev/"


#==============================================================================
# Document Class
#==============================================================================

  
class Graph:
    """ unl := Graph(link*) """
    
    def __init__(self, link_list):
        self.link_list = link_list
    
    def to_string(self):
        res = "Graph("
        res_init_length = len(res) 
        for link in self.link_list:
            res += link.to_string() + ", "
        if len(res) > res_init_length:
            res = res[:-2]
        res += ")"
        return res
    
    def to_dot_graph(self):
        graph = dot.Dot("My_Graph", graph_type="graph", bgcolor="yellow")
        for link in self.link_list:
            graph.add_node(link.uw1.to_dot_node())
            graph.add_node(link.uw2.to_dot_node())
            graph.add_edge(link.to_dot_edge())
        return graph
    
    def to_rdf_graph(self):
        graph = urdf.init_unl_rdf_graph()
        for link in self.link_list:
            graph = link.to_rdf_graph(graph) 
        return graph
    
    
class Link:
    """ link := Link(relation, uw, uw) """
    
    def __init__(self, relation, uw1, uw2):
        self.relation = relation
        self.uw1 = uw1  
        self.uw2 = uw2
    
    def to_string(self):
        res = self.relation.to_string() + "("
        res += self.uw1.to_string() + ", "
        res += self.uw2.to_string() + ")"
        return res
    
    def to_dot_edge(self):
        n1_name = self.uw1.to_dot_node().get_name()
        n2_name = self.uw2.to_dot_node().get_name()
        edge = dot.Edge(n1_name, n2_name, color="blue")
        return edge
    
    def to_rdf_graph(self, graph):
        graph, rel_node = self.relation.to_rdf_graph(graph)
        graph, uw1_node = self.uw1.to_rdf_graph(graph)
        graph, uw2_node = self.uw2.to_rdf_graph(graph)
        graph = urdf.add_relation_triple(graph, rel_node, uw1_node, uw2_node)
        return graph
    
class Relation:
    """ relation := Relation(id) """
    
    def __init__(self, id):
        self.id = id    
    
    def to_string(self):
        return self.id
    
    def to_rdf_graph(self, graph):
        return urdf.add_relation_node(graph, self.id)
    
    
class UniversalWord:
    """ uw := UniversalWord(headword, restriction*, attr*) """
    
    def __init__(self, headword, restriction_list, attr_list):
        self.headword = headword    
        self.restriction_list = restriction_list   
        self.attr_list = attr_list   
    
    def to_string(self):
        res = self.headword.to_string() + "("
        res_init_length = len(res) 
        for restriction in self.restriction_list:
            res += restriction.to_string() + ", "
        res = res[:-1]
        if len(res) > res_init_length:
            res = res[:-1]
            res += ")"
        for attr in self.attr_list:
            res += ".@" + attr.to_string()
        return res
    
    def to_id(self):
        res = self.headword.to_string()
        for restriction in self.restriction_list:
            res += "-" + restriction.to_id()
        return res        
    
    def to_dot_node(self):
        node_name = self.headword.to_string() # TODO: à revoir pour id unique
        node_label = self.headword.to_string()
        node = dot.Node(node_name, label=node_label)
        return node
    
    def to_rdf_graph(self, graph):
        return urdf.add_uw_node(graph, self.to_id(), self.to_string())
    
    
class Headword:
    """ headword := Headword(id) """
    
    def __init__(self, id):
        self.id = id    
    
    def to_string(self):
        return self.id.to_string()
    
    
class Restriction:
    """ restriction := Restriction(id) """
    
    def __init__(self, relation, id):
        self.relation = relation
        self.id = id
    
    def to_string(self):
        return self.relation.to_string() + ">" + self.id.to_string()
    
    def to_id(self):
        return self.relation.to_string() + "-" + self.id.to_string()
    
    
class Attribute:
    """ attr := Attribute(id) """
    
    def __init__(self, id):
        self.id = id    
    
    def to_string(self):
        return self.id.to_string()
    
    
class Ident:
    """ id := Ident(string) """
    
    def __init__(self, id):
        self.id = id    
    
    def to_string(self):
        return self.id
    
