#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# unlAnt: uRDF
#------------------------------------------------------------------------------
# This module contains methods to generate an UNL-RDF graph
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

import rdflib as rdf
from rdflib import Namespace, URIRef, Literal  #, BNode
from rdflib import XSD, RDFS, RDF, OWL


#==============================================================================
# Parameters
#==============================================================================

# -- Schémas, URI
UNL_RDF_SCHEMA = "https://unl.tetras-libre.fr/rdf/schema/"
UNL_RDF_BASE_URI = "https://unl.tetras-libre.fr/rdf/dev/"

# -- Namespace(s)
URDF = Namespace(UNL_RDF_SCHEMA)


#==============================================================================
# Main Functions
#==============================================================================

# ----------------------------------------------
# Methods to initialize the UNL-RDF graph
# ----------------------------------------------

def init_unl_rdf_graph():
    graph = rdf.Graph()
    graph.bind("urdf", UNL_RDF_SCHEMA) # prefix for UNL-RDF schema
    return graph


def add_useful_prefix(graph):  
    """ Add useful prefix, as RDFS, OWL... """
    
    graph.bind("xsd", XSD)
    graph.bind("rdfs", RDFS)
    graph.bind("owl", OWL)
    graph.bind("rdf", RDF)
    
    return graph


# ----------------------------------------------
# Methods to add a node in the UNL-RDF graph
# ----------------------------------------------
          
def add_relation_node(graph, relation_id):
    """ Add a node corresponding to a relation in UNL. 
    
    Attrs:
    - relation_id (str): relation identifier
    
    Returns:
    - a RDF node corresponding to the UNL relation
    """
    
    relation_node = URIRef(UNL_RDF_BASE_URI + relation_id)
    relation_label = Literal(relation_id)
    graph.add((relation_node, RDF.type, URDF.UW_Occurrence))
    graph.add((relation_node, RDFS.label, relation_label))
    
    return graph, relation_node


def add_uw_node(graph, uw_id, uw_label):
    """ Add a node corresponding to an universal word (UW) in UNL. 
    
    Attrs:
    - uw_id (str): UW identifier
    - uw_label (str): label for the UW node
    
    Returns:
    - a RDF node corresponding to the UW
    """
    
    uw_node = URIRef(UNL_RDF_BASE_URI + uw_id)
    uw_label = Literal(uw_label)
    graph.add((uw_node, RDF.type, URDF.UW_Occurrence))
    graph.add((uw_node, RDFS.label, uw_label))
    
    return graph, uw_node
      
    
# ----------------------------------------------
# Methods to add a triple in the UNL-RDF graph
# ----------------------------------------------    
    
def add_relation_triple(graph, relation_node, uw1_node, uw2_node):
    """ Add a triple corresponding to a relation between two nodes in UNL. 
    
    Attrs:
    - relation_node (RDF node): relation node
    - uw1_node (RDF node): UW1 node
    - uw2_node (RDF node): UW2 node
    """
    
    graph.add((uw1_node, relation_node, uw2_node))
    
    return graph

    
    
