//=============================================================================
// ANTLR Grammar for UNL Document
//=============================================================================

grammar unl;

@header {
from asd import unl
}


//=============================================================================
// Parser Grammar
//=============================================================================

//---------------------------------------------------------
// UNL representation
//---------------------------------------------------------

unlGraph returns [out]
  : '{unl}'                               {link_list = []}
    (l=link                               {link_list.append($l.out)} )+
    '{/unl}'                              {$out = unl.Graph(link_list)}
  ;

link returns [out]
  : r=universalRelation
    LP uw1=universalWord COMMA uw2=universalWord RP
                                          {$out = unl.Link($r.out, $uw1.out, $uw2.out)}
  ;

universalWord returns [out]
  : h=headword                            {headword =  $h.out}
                                          {restriction_list = []}
    (LP
    r1=restriction                        {restriction_list.append($r1.out)}
    (COMMA r2=restriction                 {restriction_list.append($r2.out)} )*
    RP)?
                                          {attr_list = []}
    (a=attribute                          {attr_list.append($a.out)} )*
                                          {$out = unl.UniversalWord(headword, restriction_list, attr_list)}
  | v=value                               {$out = unl.UniversalWord($v.out, [], [])}
  ;

headword returns [out]
  : i=ident                              {$out = unl.Headword($i.out)}
  ;

restriction returns [out]
  : r=universalRelation GREATER i=ident  {$out = unl.Restriction($r.out, $i.out)}
  ;

attribute returns [out]
  : DOT AT i=ident                       {$out = unl.Attribute($i.out)}
  ;

value returns [out]
  : v=VALUE                               {$out = unl.Ident($v.text)}
  ;

universalRelation returns [out]
  : r=( AND | AOJ | BEN | CNT
      | EQU | ICL | OBJ | QUA )           {$out = unl.Relation($r.text)}
  ;


//---------------------------------------------------------
// Base Element
//---------------------------------------------------------

sentence : (word | punctuation | bracket)* ;

ident returns [out]
  : w1=word                               {ident = $w1.out}
    (UNDERSCORE w2=word                   {ident += "_" + $w2.out} )*
                                          {$out = unl.Ident(ident)}
  ;

word returns [$out]
  : l=LETTER                              {$out = $l.text}
  | w=WORD                                {$out = $w.text}
  ;

punctuation : DOT | COMMA | SEMCOL | COLON | DASH ;

bracket : LP | RP | LC | RC ;


//=============================================================================
// Lexer Grammar
//=============================================================================

// ignore whitespaces
WS              : (' '|'\n'|'\t'|'\r'|'\u000C')+ -> skip ;

// fragments
fragment LOWERCASE : [a-z] ;
fragment UPPERCASE : [A-Z] ;
fragment DIGIT  : '0'..'9' ;
fragment ASCII  : ~('\n'|'"'|'<'|'>'|'('|')') ;

// punctuation
DOT             : '.' ;
COMMA           : ',' ;
SEMCOL          : ';' ;
COLON           : ':' ;
DASH            : '-' ;

// brackets
LP              : '(' ; // Left parenthesis
RP              : ')' ;
LC              : '{' ; // Left curly bracket
RC              : '}' ;

// symbols
LESS            : '<' ;
GREATER         : '>' ;
AT              : '@' ;
UNDERSCORE      : '_' ;

// relations
AND             : 'and' ;
AOJ             : 'aoj' ;
BEN             : 'ben' ;
CNT             : 'cnt' ;
EQU             : 'equ' ;
ICL             : 'icl' ;
OBJ             : 'obj' ;
QUA             : 'qua' ;

// other tokens
LETTER          : LOWERCASE | UPPERCASE ;
WORD            : (LETTER)+ ;
VALUE           : (DIGIT)+ (DOT (DIGIT)+)? ;

