#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# unlAnt: parse
#------------------------------------------------------------------------------
# Main script to parse an UNL document file  
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

import sys
from subprocess import Popen, PIPE
from antlr4 import FileStream, CommonTokenStream, InputStream


#==============================================================================
# Parameters
#==============================================================================

# ANTLR Grammar
doc_grammar = 'grammar/doc/doc.g4'
org_grammar = 'grammar/org/org.g4'
unl_grammar = 'grammar/unl/unl.g4'

# RDF URIs
UNL_RDF_BASE_URI = "https://unl.tetras-libre.fr/rdf/dev/"


#==============================================================================
# Utilities
#==============================================================================

def run_command(cmd):
    with Popen(cmd, stdout=PIPE, stderr=PIPE, universal_newlines=True) as p:
        p.poll()
        p.stdout.flush()
        p.stderr.flush()
        stdout, stderr = p.communicate()
    return p.returncode, stdout, stderr

     
#==============================================================================
# Parsing Functions
#==============================================================================

def create_lexer_parser_with_antlr(grammar_file):
    """ Create python lexer and parser using ANTLR4 """

    cmd = ['antlr4', '-Dlanguage=Python3', grammar_file]   
    print("--- Create python lexer and parser (run command: " + str(cmd) + ")")
    code, out, err = run_command(cmd)
    if code != 0:
        print("Error in grammar: \n\n" + err)
        
        
def instantiate_lexer_parser(input, antLexer, antParser):
    """ Instantiate lexer and parser """
    
    print("--- Instantiate lexer and parser")
    lexer = antLexer(input)
    stream = CommonTokenStream(lexer)
    parser = antParser(stream)
    
    return parser
        
    
def parse_document(input):
    
    # -- Create python lexer and parser
    create_lexer_parser_with_antlr(doc_grammar)
    
    # -- Import Lexer/Parser (after creation by ANTLR4)
    from grammar.doc.docLexer import docLexer
    from grammar.doc.docParser import docParser
    
    # -- Parse document
    parser = instantiate_lexer_parser(input, docLexer, docParser)
    print("--- Parse document to separate org part and unl part")
    tree = parser.document()
    print("----- resulting tree:\n" + tree.toStringTree(recog=parser))    
    document = tree.out
    
    return document
    

def parse_org(input):
    
    # -- Create python lexer and parser
    create_lexer_parser_with_antlr(org_grammar)
    
    # -- Import Lexer/Parser (after creation by ANTLR4)
    from grammar.org.orgLexer import orgLexer
    from grammar.org.orgParser import orgParser
    
    # -- Parse UNL part
    parser = instantiate_lexer_parser(input, orgLexer, orgParser)
    print("--- Parse origin sentence")
    tree = parser.orgPart()
    print("----- resulting tree:\n" + tree.toStringTree(recog=parser))
    

def parse_unl(input):
    
    # -- Create python lexer and parser
    create_lexer_parser_with_antlr(unl_grammar)
    
    # -- Import Lexer/Parser (after creation by ANTLR4)
    from grammar.unl.unlLexer import unlLexer
    from grammar.unl.unlParser import unlParser
    
    # -- Parse UNL part
    parser = instantiate_lexer_parser(input, unlLexer, unlParser)
    print("--- Parse UNL representation")
    tree = parser.unlGraph()
    print("----- resulting tree:\n" + tree.toStringTree(recog=parser))  
    unl = tree.out
    
    return unl
    
     
#==============================================================================
# Main Function
#============================================================================== 

def main(argv):
    
    # -- Read input file
    input_file = argv[1]
    input = FileStream(input_file)
    
    # -- Document Parsing
    print("-- Document Parsing ")
    document = parse_document(input)
    org_part = document.sentence.org_part.to_string()
    unl_part = document.sentence.unl_part.to_string()
    print("----- org_part:\n" + org_part)
    print("----- unl_part:\n" + unl_part)
    
    # -- ORG Parsing (Sentence Original Part)
    print("-- ORG Parsing (Origin Sentence) ")
    parse_org(InputStream(org_part)) 
    
    # -- UNL Parsing (Sentence UNL Part)
    print("-- UNL Parsing (UNL Representation) ")
    unl = parse_unl(InputStream(unl_part))
    print("----- UNL string:\n" + unl.to_string())
    
    # -- Generate dot file
    unl_dot_graph = unl.to_dot_graph()
    print("----- DOT string:\n" + unl_dot_graph.to_string())
    unl_dot_graph.write_raw("output/output_raw.dot")
    unl_dot_graph.write_png("output/output.png")
    
    # -- Generate rdf file
    unl_rdf_graph = unl.to_rdf_graph()
    print("----- RDF file generated: " + "output/output.ttl")
    unl_rdf_graph.serialize(destination ="output/output.ttl", 
                            base = UNL_RDF_BASE_URI + "output", 
                            format = 'turtle')


if __name__ == '__main__':
    main(sys.argv)
